

#include <XConfig.h>
#include <XOpenGLView.h>
#include <XWindow.h>
#include <XDraw.h>

#if OPT_XWIN
#include <string.h>
#include <stdio.h>
#endif

XGOpenGLView *XGOpenGLView::sCurrentCtx = 0;

XGOpenGLView::XGOpenGLView( XGView *parent, XGArgStream &s ):
	XGView( parent, s )
{
#if OPT_MACOS
	fCtx = 0;
#endif
#if OPT_WINOS
	fHWND = 0;
	fDC = 0;
	fGLRC = 0;
	fWindowShown = false;
#endif
#if OPT_XWIN
	memset(&GLWin, 0, sizeof(GLWin));
#endif
	fBackBuffer = !s.ReadBoolean();
	fDepthBuffer = !s.ReadBoolean();
}



XGOpenGLView::~XGOpenGLView()
{
	if (GetCurrentContext() == this) {
		SetCurrentContext( 0 );
	}
	
#if OPT_MACOS
	if (fCtx) {
		aglSetDrawable( fCtx, 0 );
		aglDestroyContext( fCtx );
		fCtx = 0;
	}
#endif

#if OPT_WINOS
	if (fDC) {
		ReleaseDC( fHWND, fDC );
		fDC = 0;
	}
	
	if (fGLRC) {
		wglDeleteContext( fGLRC );
		fGLRC = 0;
	}
	
	if (fHWND) {
		DestroyWindow( fHWND );
		fHWND = 0;
	}
#endif

/* -- ALAN: clean up -- */
#if OPT_WXIN
    if (GLWin.ctx)
    {
		 /* -- disconnect the GLX context from the window -- */
		glXMakeCurrent(GLWin.dpy, None, NULL);

		/* -- destroy the GLX context -- */
        glXDestroyContext(GLWin.dpy, GLWin.ctx);
        GLWin.ctx = NULL;
    }
#endif // OPT_XWIN
}


void XGOpenGLView::DoDrawView( XRect )
{
	XGOpenGLDraw draw( this );	// set clip region, etc.
	
	glClearColor( 1, 0, 0, 0 );
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glBegin(GL_POLYGON);
	glColor3d(1.0, 1.0, 0.0);
	glVertex3d( 0.8,  0.8, 0.0);
	glColor3d(0.0, 1.0, 0.0);
	glVertex3d( 0.8, -0.8, 0.0);
	glColor3d(0.0, 0.0, 1.0);
	glVertex3d(-0.8, -0.8, 0.0);
	glColor3d(1.0, 0.0, 1.0);
	glVertex3d(-0.8,  0.8, 0.0);
	glEnd();
	glFinish();
	this->ShowBackBuffer();
}


void XGOpenGLView::DoSizeView()
{
	XGView::DoSizeView();
	UpdateViewport();
}


void XGOpenGLView::DoMoveView()
{
	XGView::DoMoveView();
	UpdateViewport();
}


#if OPT_WINOS
void XGOpenGLView::DoShowHide()
{
	if (fHWND && fWindowShown && !IsVisible()) {
		ShowWindow( fHWND, SW_HIDE );
		fWindowShown = false;
	}
}
#endif


void XGOpenGLView::ShowBackBuffer()
{
#if OPT_MACOS
	if (fCtx && fBackBuffer) {
		aglSwapBuffers( this->fCtx );
	}
#endif
#if OPT_WINOS
	if (fDC && fGLRC) {	
		SwapBuffers( fDC );
	}
#endif

/* -- ALAN: blow this stuff to the screen -- */
#if OPT_XWIN
    if (GLWin.doubleBuffered)
    {
        glXSwapBuffers(GLWin.dpy, GLWin.win);
    }
#endif // OPT_XWIN
}


#if OPT_MACOS
void XGOpenGLView::InitOpenGLView( bool doublebuf, bool depthbuf )
{
	if (fCtx) return;
	
	aglConfigure( AGL_RETAIN_RENDERERS, GL_TRUE );	// Build 100 - PAB - M3 always doesn't see so we will also
	
	GLint attrib[10];
	int attrib_count = 0;
	attrib[attrib_count++] = AGL_ALL_RENDERERS;
	attrib[attrib_count++] = AGL_RGBA;
	if (doublebuf) {
		attrib[attrib_count++] = AGL_DOUBLEBUFFER;
	}
	if (depthbuf) {
		attrib[attrib_count++] = AGL_DEPTH_SIZE;
		attrib[attrib_count++] = 16;
	}
	attrib[attrib_count++] = AGL_NONE;
	
	AGLPixelFormat fmt = aglChoosePixelFormat( NULL, 0, attrib );
	if (!fmt) return;
	
	AGLContext ctx = aglCreateContext( fmt, NULL );
	if (!ctx) {
		aglDestroyPixelFormat( fmt );
		return;
	}
	
	if (!aglSetDrawable( ctx, (CGrafPtr)GetWindow()->GetGrafPtr() )) {
		aglDestroyContext( ctx );
		aglDestroyPixelFormat( fmt );
		return;
	}
	
	aglDestroyPixelFormat( fmt );
	fCtx = ctx;
	
	if (depthbuf) {
		XGOpenGLView *old_ctx = GetCurrentContext();
		SetCurrentContext( this );
		glEnable( GL_DEPTH_TEST );
		SetCurrentContext( old_ctx );
	}
	UpdateViewport();
}
#endif


#if OPT_WINOS
static PIXELFORMATDESCRIPTOR sPFD = { 
    sizeof(PIXELFORMATDESCRIPTOR),    // size of this pfd 
    1,                                // version number 
    PFD_DRAW_TO_WINDOW |              // support window 
    PFD_SUPPORT_OPENGL,               // support OpenGL 
    PFD_TYPE_RGBA,                    // RGBA type 
    24,                               // 24-bit color depth 
    0, 0, 0, 0, 0, 0,                 // color bits ignored 
    0,                                // no alpha buffer 
    0,                                // shift bit ignored 
    0,                                // no accumulation buffer 
    0, 0, 0, 0,                       // accum bits ignored 
    32,                               // 32-bit z-buffer     
    0,                                // no stencil buffer 
    0,                                // no auxiliary buffer 
    PFD_MAIN_PLANE,                   // main layer 
    0,                                // reserved 
    0, 0, 0                           // layer masks ignored 
};

ATOM XGOpenGLView::sWindowClass = 0;

LRESULT CALLBACK XGOpenGLView::WndProc(
    HWND hwnd,        // handle to window
    UINT uMsg,        // message identifier
    WPARAM wParam,    // first message parameter
    LPARAM lParam)    // second message parameter
{ 
 
    switch (uMsg) 
    { 
        case WM_CREATE: 
            // Initialize the window. 
            return 0; 
 
        case WM_PAINT: 
            // Paint the window's client area. 
            return 0; 
 
        case WM_SIZE: 
            // Set the size and position of the window. 
            return 0; 
 
        case WM_DESTROY: 
            // Clean up window-specific data objects. 
            return 0; 
 
        // 
        // Process other messages. 
        // 
 
        default: 
            return DefWindowProc(hwnd, uMsg, wParam, lParam); 
    } 
    return 0; 
} 

void XGOpenGLView::InitOpenGLView( bool doublebuf, bool depthbuf )
{
	if (fHWND || fDC || fGLRC) return;
	
	if (!sWindowClass) {
		WNDCLASS wc;
		wc.style = CS_HREDRAW | CS_OWNDC;
		wc.lpfnWndProc = XGOpenGLView::WndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;
		wc.hInstance = _GInstance;
		wc.hIcon = NULL;
		wc.hCursor = NULL;
		wc.hbrBackground = NULL;//CreateSolidBrush(PALETTERGB(0, 0, 255));
		wc.lpszMenuName = NULL;
		wc.lpszClassName = "XGOpenGLView";
		sWindowClass = RegisterClass( &wc );
		if (!sWindowClass) return;
	}
	
	XRect r = GetContentRect();
	ViewToGlobal( &r );
	fHWND = ::CreateWindow( (char *)sWindowClass,	// class
							"",			// title
							WS_CHILD | WS_VISIBLE | WS_DISABLED | WS_CLIPSIBLINGS,	// style
							r.left,
							r.top,
							r.right - r.left,
							r.bottom - r.top,
							GetRootWindow(),
							NULL,			// menu
							_GInstance,
							NULL);			// data
	if (!fHWND) return;
	fWindowShown = true;
	
	PIXELFORMATDESCRIPTOR pfd;
	memcpy( &pfd, &sPFD, sizeof(PIXELFORMATDESCRIPTOR) );
	if (doublebuf) {
		pfd.dwFlags |= PFD_DOUBLEBUFFER;
	}
	if (!depthbuf) {
		pfd.cDepthBits = 0;
		pfd.dwFlags |= PFD_DEPTH_DONTCARE;
	}
	
	fDC = GetDC( fHWND );
	if (fDC) {
		int iPixelFormat = ChoosePixelFormat( fDC, &pfd );
		if (iPixelFormat && SetPixelFormat( fDC, iPixelFormat, &pfd )) {
			fGLRC = wglCreateContext( fDC );
		}
		if (!fGLRC) {
			ReleaseDC( fHWND, fDC );
			fDC = 0;
		}
	}
	
	if (depthbuf) {
		XGOpenGLView *old_ctx = GetCurrentContext();
		SetCurrentContext( this );
		glEnable( GL_DEPTH_TEST );
		SetCurrentContext( old_ctx );
	}
	UpdateViewport();
}
#endif


/************************************************************************/
/*																		*/
/*	ALAN: XGOpenGLView (XWin)											*/
/*																		*/
/************************************************************************/
/* -- NOTE: we are using the OpenGL extension to X (GLX) -- */
#if OPT_XWIN

/* -- Set us up the best visual we can get, suitable for GL -- */
XVisualInfo *
XGOpenGLView::GetGLVisual (Display *dpy, int screen)
{

# define R GLX_RED_SIZE
# define G GLX_GREEN_SIZE
# define B GLX_BLUE_SIZE
# define D GLX_DEPTH_SIZE
# define I GLX_BUFFER_SIZE
# define DB GLX_DOUBLEBUFFER
# define ST GLX_STENCIL_SIZE

  int attrs[][20] = {
   { GLX_RGBA, R, 8, G, 8, B, 8, D, 8, DB, ST,1, 0 }, /* rgb double, stencil */
   { GLX_RGBA, R, 4, G, 4, B, 4, D, 4, DB, ST,1, 0 },
   { GLX_RGBA, R, 2, G, 2, B, 2, D, 2, DB, ST,1, 0 },
   { GLX_RGBA, R, 8, G, 8, B, 8, D, 8, DB,       0 }, /* rgb double */
   { GLX_RGBA, R, 4, G, 4, B, 4, D, 4, DB,       0 },
   { GLX_RGBA, R, 2, G, 2, B, 2, D, 2, DB,       0 },
   { GLX_RGBA, R, 8, G, 8, B, 8, D, 8,           0 }, /* rgb single */
   { GLX_RGBA, R, 4, G, 4, B, 4, D, 4,           0 },
   { GLX_RGBA, R, 2, G, 2, B, 2, D, 2,           0 },
   { I, 8,                       D, 8, DB,       0 }, /* cmap double */
   { I, 4,                       D, 4, DB,       0 },
   { I, 8,                       D, 8,           0 }, /* cmap single */
   { I, 4,                       D, 4,           0 },
   { GLX_RGBA, R, 1, G, 1, B, 1, D, 1,           0 }  /* monochrome */
  };

  int i;
  for (i = 0; i < sizeof(attrs)/sizeof(*attrs); i++)
    {
      XVisualInfo *vi = glXChooseVisual (dpy, screen, attrs[i]);
      if (vi)
        {
           return vi;
        }
    }

  return 0;
}

/* -- Set us up our GLX window, attached to parent dialog -- */
void XGOpenGLView::InitOpenGLView( bool doublebuf, bool depthbuf )
{
	if ( GLWin.win || GLWin.ctx ) return;

    XVisualInfo *vi;
    Colormap cmap;

    GLWin.dpy = _GDisplay;
    GLWin.screen = DefaultScreen(GLWin.dpy);


	/* -- get our visual -- */
	vi = GetGLVisual( GLWin.dpy, GLWin.screen );

	if( vi != NULL )
	{
		int value=0;
		int status=0;

		status = glXGetConfig (GLWin.dpy, vi, GLX_USE_GL, &value);
		if (status == GLX_NO_EXTENSION) {
			printf("dpy does not support the GLX extension.\n");
			return;
		}
		if (status == GLX_BAD_VISUAL || value == false) {
			printf("this visual does not support GLX.\n");
			return;
		}


		if (!glXGetConfig (GLWin.dpy, vi, GLX_DOUBLEBUFFER, &value))
			GLWin.doubleBuffered = value;

		if (!glXGetConfig (GLWin.dpy, vi, GLX_DEPTH_SIZE, &value))
			GLWin.depth = value;

#if 0
		int major_version, minor_version;
		glXQueryVersion(GLWin.dpy, &major_version, &minor_version);
		printf ("	GLX version:		%d.%d\n", major_version, minor_version);

		printf ("	GLX depth size:		%d\n", GLWin.depth);
		printf ("	GLX Dbl buffered:	%s\n", GLWin.doubleBuffered ? "Yes" : "No");

		if (!glXGetConfig (GLWin.dpy, vi, GLX_RGBA, &value) && value)
		  {
			int r=0, g=0, b=0, a=0;
			glXGetConfig (GLWin.dpy, vi, GLX_RED_SIZE,   &r);
			glXGetConfig (GLWin.dpy, vi, GLX_GREEN_SIZE, &g);
			glXGetConfig (GLWin.dpy, vi, GLX_BLUE_SIZE,  &b);
			glXGetConfig (GLWin.dpy, vi, GLX_ALPHA_SIZE, &a);
			printf ("	GLX type:		RGBA (%2d, %2d, %2d, %2d)\n",
					r, g, b, a);

			r=0, g=0, b=0, a=0;
			glXGetConfig (GLWin.dpy, vi, GLX_ACCUM_RED_SIZE,   &r);
			glXGetConfig (GLWin.dpy, vi, GLX_ACCUM_GREEN_SIZE, &g);
			glXGetConfig (GLWin.dpy, vi, GLX_ACCUM_BLUE_SIZE,  &b);
			glXGetConfig (GLWin.dpy, vi, GLX_ACCUM_ALPHA_SIZE, &a);
			printf ("	GLX accum:		RGBA (%2d, %2d, %2d, %2d)\n",
					r, g, b, a);
		  }
		else
		  {
			value = 0;
			glXGetConfig (GLWin.dpy, vi, GLX_BUFFER_SIZE, &value);
			printf ("	GLX type:			indexed (%d)\n", value);
		  }
#endif
	}
	else {
		printf("failed to get a visual\n"); // fatal error
		return;
	}


    /* -- create a GLX context -- */
    GLWin.ctx = glXCreateContext(GLWin.dpy, vi, 0, GL_TRUE);

	if( GLWin.ctx == NULL ) {
		printf("Error on CreateContext\n"); // fatal error
		return;
	}

#if 0
	printf ("	Direct Rendering:	%s\n", glXIsDirect(GLWin.dpy, GLWin.ctx) ? "Yes" : "No");
#endif

	/* -- get our window position -- */
	XRect r = GetContentRect();
	ViewToGlobal( &r );

    /* -- create a color map -- */
	/* -- I may want to rethink this, use DefaultColormap instead -- */
    cmap = ::XCreateColormap(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
        vi->visual, AllocNone);

	/* -- set our window attributes -- */
    GLWin.attr.colormap = cmap;
    GLWin.attr.border_pixel = 0;


	/* -- create a window in windowed mode with no event mask -- */
	/* -- I don't want an event loop, parent handles messages -- */
	GLWin.win = ::XCreateWindow(GLWin.dpy, GetRootWindow(),
            r.left, r.top, r.right - r.left, r.bottom - r.top, 0, vi->depth, InputOutput, vi->visual,
            CWBorderPixel | CWColormap, &GLWin.attr);


	XFree ((char*)vi);


#if 0
	Atom wmDelete;

	/* -- only set window title and handle wm_delete_events if in windowed mode -- */
	wmDelete = ::XInternAtom(GLWin.dpy, "WM_DELETE_WINDOW", true);
	::XSetWMProtocols(GLWin.dpy, GLWin.win, &wmDelete, 1);
	::XSetStandardProperties(GLWin.dpy, GLWin.win, "xogl",
            "xogl", None, NULL, 0, NULL);
#endif

	/* -- mark our new window for display -- */
	::XMapRaised(GLWin.dpy, GLWin.win);
 // ::XMapWindow(GLWin.dpy, GLWin.win);

        
    /* -- connect the glx-context to the window -- */
    if( glXMakeCurrent(GLWin.dpy, GLWin.win, GLWin.ctx) == GL_FALSE )
		printf("Error MakeCurrent\n"); // fatal error

#if 0
	Window winDummy;
	unsigned int borderDummy;

	/* -- record our position ... do I need this? -- */
    ::XGetGeometry(GLWin.dpy, GLWin.win, &winDummy, &GLWin.x, &GLWin.y,
        &GLWin.width, &GLWin.height, &borderDummy, &GLWin.depth);
#endif
	
	/* -- enable gl_depth -- */
	if (depthbuf) {
		XGOpenGLView *old_ctx = GetCurrentContext();
		SetCurrentContext( this );
		glEnable( GL_DEPTH_TEST );
		SetCurrentContext( old_ctx );
	}

	/* -- set the viewport -- */
	UpdateViewport();
}
#endif // OPT_XWIN


void XGOpenGLView::UpdateViewport()
{
#if OPT_MACOS
	if (fCtx) {
		XGOpenGLView *old_ctx = GetCurrentContext();
		SetCurrentContext( this );
		XRect r = GetContentRect();
		ViewToGlobal( &r );
		XRect w = GetWindow()->GetContentRect();
		glViewport( r.left, w.bottom - r.bottom, r.right - r.left, r.bottom - r.top );
		SetCurrentContext( old_ctx );
	}
#endif

#if OPT_WINOS
	if (fHWND && fDC && fGLRC) {
		XRect r = GetContentRect();
		ViewToGlobal( &r );
		SetWindowPos( fHWND, NULL,
					r.left, r.top, r.right-r.left, r.bottom - r.top,
					SWP_NOZORDER );
		XGOpenGLView *old_ctx = GetCurrentContext();
		SetCurrentContext( this );
		glViewport( 0, 0, r.right - r.left, r.bottom - r.top );
		SetCurrentContext( old_ctx );
	}
#endif

/* -- ALAN:  set the viewport -- */
#if OPT_XWIN
	if ( GLWin.dpy && GLWin.win && GLWin.ctx )
	{
		XRect r = GetContentRect();
		ViewToGlobal( &r );
		::XResizeWindow(GLWin.dpy,
						GLWin.win,
						r.right - r.left,
						r.bottom - r.top);
		XGOpenGLView *old_ctx = GetCurrentContext();
		SetCurrentContext( this );
		glViewport( 0, 0, r.right - r.left, r.bottom - r.top );
		SetCurrentContext( old_ctx );
	}
#endif // OPT_XWIN
}


XGOpenGLView *XGOpenGLView::GetCurrentContext()
{
	return sCurrentCtx;
}


void XGOpenGLView::SetCurrentContext( XGOpenGLView *ctx )
{
	if (ctx == sCurrentCtx) return;
#if OPT_MACOS
	if (ctx && ctx->fCtx) {
		if (aglSetCurrentContext(ctx->fCtx)) {
			sCurrentCtx = ctx;
		}
	} else {
		if (aglSetCurrentContext( 0 )) {
			sCurrentCtx = 0;
		}
	}
#endif

#if OPT_WINOS
	if (ctx && ctx->fHWND && ctx->fDC && ctx->fGLRC) {
		if (wglMakeCurrent( ctx->fDC, ctx->fGLRC )) {
			sCurrentCtx = ctx;
		}
	} else {
		if (wglMakeCurrent( NULL, NULL )) {
			sCurrentCtx = 0;
		}
	}
#endif

/* -- ALAN:  attach GLX context to the window  -- */
#if OPT_XWIN
	if( ctx && 	ctx->GLWin.win && ctx->GLWin.ctx ) {
		if( glXMakeCurrent(ctx->GLWin.dpy, ctx->GLWin.win, ctx->GLWin.ctx) == GL_TRUE ) {
			sCurrentCtx = ctx;
		}
	} else {
		if( ctx ) {
			glXMakeCurrent(ctx->GLWin.dpy, GLX_NONE, NULL);
		}
		sCurrentCtx = 0;
	}
#endif // OPT_XWIN
}



XGOpenGLDraw::XGOpenGLDraw( XGOpenGLView *v ):
	XGDraw( v )
{
#if OPT_MACOS
	if (!v->fCtx) {
		v->InitOpenGLView( v->fBackBuffer, v->fDepthBuffer );
	}
#endif

#if OPT_WINOS
	if (!v->fHWND) {
		v->InitOpenGLView( v->fBackBuffer, v->fDepthBuffer );
	}
	if (v->fHWND && !v->fWindowShown && v->IsVisible()) {
		::ShowWindow( v->fHWND, SW_SHOW );
		v->fWindowShown = true;
	}
#endif
	
/* -- ALAN:  set us up the GLX window  -- */
#if OPT_XWIN
	if( !v->GLWin.win ) {
		v->InitOpenGLView( v->fBackBuffer, v->fDepthBuffer );
	}
#endif // OPT_XWIN

	old_ctx = XGOpenGLView::GetCurrentContext();
	XGOpenGLView::SetCurrentContext( v );
}


XGOpenGLDraw::~XGOpenGLDraw()
{
	XGView *v = GetView();
	if (v) {
		XGOpenGLView *gv = static_cast<XGOpenGLView *>(v);
		glFinish();
		if (gv->fBackBuffer) {
			gv->ShowBackBuffer();
		}
	}
	XGOpenGLView::SetCurrentContext( old_ctx );
}



