

#ifndef __XGLVIEW_H__
#define __XGLVIEW_H__

#ifndef PandaWave_XVIEW
#include <XView.h>
#endif

#ifndef PandaWave_XDRAW
#include <XDraw.h>
#endif

#if OPT_MACOS && !defined(_AGL_H)
#include <agl.h>
#else
	#if OPT_XWIN
		#include <GL/gl.h>
		#include <GL/glx.h>
	#else
		#include <gl.h>
	#endif
#endif


class XGOpenGLView: public XGView {
	public:
		friend class		XGOpenGLDraw;
		
							XGOpenGLView( XGView *parent, XGArgStream &s );
		virtual				~XGOpenGLView();
		
		enum {
			kClassID = 'opgl'
		};
		
		void				DoDrawView( XRect );
		void				DoSizeView();
		void				DoMoveView();
	#if OPT_WINOS
		void				DoShowHide();
	#endif
		
		void				ShowBackBuffer();
		
		
		static XGOpenGLView	*GetCurrentContext();
		static void			SetCurrentContext( XGOpenGLView * );
		
	protected:
		static XGOpenGLView	*sCurrentCtx;
		
		void 				InitOpenGLView( bool doublebuffer = true,
											bool depthbuffer = true );
		virtual void		UpdateViewport();
		
		bool				fBackBuffer;
		bool				fDepthBuffer;
#if OPT_MACOS
		AGLContext			fCtx;
#endif
#if OPT_WINOS
		HWND				fHWND;
		HDC					fDC;
		HGLRC				fGLRC;
		bool				fWindowShown;
		static LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM );
		static ATOM			sWindowClass;
#endif

/* -- ALAN: Define our window structure -- */
#if OPT_XWIN
/* 
 * Description of Structure Members:
 *
 * Display 				*dpy: 	This is the display. In X Windows, a display considers
 *								everything. Monitor, Mouse, Keyboard.
 * int	  				screen:	Which window we are rendering too ?
 * Window 				win:	This is our drawable surrface
 * GLXContext 			ctx:	Stores information about our OpenGL Context
 * XSetWindowAttributes attr:
 * int					x,y:
 * unsigned int			width:	Width of the window
 * unsigned int			height:	Height of the window
 * unsigned int 		bpp:	Bits Per Pixel. With XFree86, highest we can acheive is 24.	
 */
	typedef struct {
			Display					*dpy;
			int						screen;
			Window					win;
			GLXContext				ctx;
			XSetWindowAttributes	attr;
			bool					doubleBuffered;
			int						x, y;
			unsigned int			width, height;
			unsigned int			depth;    
		} GLWindow;

		GLWindow					GLWin;

		XVisualInfo *GetGLVisual (Display *dpy, int screen);

#endif // OPT_XWIN
};


class XGOpenGLDraw: public XGDraw {
	public:
		XGOpenGLDraw( XGOpenGLView *v );
		~XGOpenGLDraw();
	
	private:
		XGOpenGLView *old_ctx;
};


#endif

